#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <locale.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <wctype.h>
#include "agi-namelookup.h"
#include "version.h"


/*
*    Lookup telephone number AGI.
*    Required external program is w3m.
*    Tested with Asterisk and xinetd.
*
*    Author: R.J. van der Putten, Leiden, Holland, 
*    rob at sput dot nl.
* 
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License version 3 as
*    published by the Free Software Foundation.
* 
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*/


/*  Overall timeout (s)  */
#define ANL_MAX_TIM 5

/*  w3m and Url  */
#define ANL_W3M_HOME   "/var/local/log/asterisk"
//#define ANL_W3M_COMND  "w3m -cols 240 -no-cookie -o display_charset=UTF-8 -o ssl_verify_server=n -dump"
#define ANL_W3M_COMND  "w3m -cols 240 -o display_charset=UTF-8 -o ssl_verify_server=n -dump"
#define ANL_URL_PREFIX "http://www.nummerid.com/result.php?input_telefoon="
#define ANL_URL_SUFFIX "&submit_telefoon.x=52&submit_telefoon.y=18&input_naam=&input_adres=&input_postcode="

/*  Log file  */
#define ANL_LOG_PATH "/var/local/log/asterisk/"
#define ANL_LOG_FIL  "namelookup.log"
#define ANL_LOG_FILE ANL_LOG_PATH ANL_LOG_FIL

/*  Files  */
FILE *flog, *lstfil, *pweb;
int  dbfd;
void *dbaddr;
size_t dbsiz;

/*  Debug  */
int debflg;

/*  Compact files  */
int compflg;


/*  Write to syslog  */
void wrtsysl(char *str, char *str2, int mode)
{
    int prio;

    if(mode == 0)
	prio = LOG_INFO;
    else
	prio = LOG_ERR;

    openlog("namelookup", LOG_PID, LOG_USER);
    if (str2 == NULL)
	syslog(prio, "%s", str);
    else
	syslog(prio, "%s: %s", str, str2);
    closelog();
}

/*  Avoid hanging script  */
void timeout(int sig)
{
    if(lstfil != NULL)
	fclose(lstfil);
    if(dbaddr != NULL)
	munmap(dbaddr, dbsiz);
    if(dbfd != 0)
	close(dbfd);
    if(pweb != NULL)
	pclose(pweb);
    if(flog != NULL)
	fclose(flog);
    wrtsysl("Timeout expired", NULL, 1);
    exit(0);
}

/*  Write ISO date to log  */
void wrtlogtim(FILE *flog)
{
    time_t t;
    char datstr[24];
    struct tm *loc;

    t = 0;
    loc = NULL;
    memset(datstr, 0, 24);

    t = time(NULL);
    loc = localtime(&t);
    /*  YYYY-MM-DD hh:mm:ss  */
    strftime(datstr, 23, "%F %T", loc);
    fprintf(flog, "%s\t", datstr);
}

/*  Write to stdout / inetd  */
void wrtstr(char *str)
{
    int ret, slen, todo;

    slen = strlen(str);
    todo = slen;
    while (todo > 0) {
	ret = write(1, str + slen - todo, todo);
	if (ret < 0) {
	    if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK) {
		ret = 0;
	    } else {
		wrtsysl("cant write", NULL, 1);
		break;
	    }
	}
        todo = todo - ret;
    }
}

/*  Names compare  */
static int compnam(const void *n1, const void *n2)
{
    char *name1, *name2;

    name1 = (char *) n1;
    name2 = (char *) n2;

    return(strcmp(name1, name2));
}

/*  Phone numbers compare  */
static int compnum(const void *n1, const void *n2)
{
    int len;
    struct phnum *pho1, *pho2;

    pho1 = (struct phnum *) n1;
    pho2 = (struct phnum *) n2;

    /*  One of these may contain the length  */
    if(pho1->len == 0)
	len = pho2->len;
    else
	len = pho1->len;

    if (len == 0) {
	/*  Neither contains a length; Exact match  */
	return(strcmp(pho1->num, pho2->num));
    } else {
	/*  Match the first bit  */
	return(strncmp(pho1->num, pho2->num, len));
    }
}

/*  Phone numbers index compare  */
static int compidx(const void *n1, const void *n2)
{
    int len;
    struct phidx *pho1, *pho2;

    pho1 = (struct phidx *) n1;    
    pho2 = (struct phidx *) n2;

    /*  One of these may contain the length  */
    if(pho1->len == 0)
	len = pho2->len;
    else
	len = pho1->len;

    if (len == 0) {
	/*  Neither contains a length; Exact match  */
	return(strcmp(pho1->num, pho2->num));
    } else {
	/*  Match the first bit  */
	return(strncmp(pho1->num, pho2->num, len));
    }    
}

/*  Lookup in list file  */
int idxlookup(int offs, int len, char *name, int mode)
{
    int ret;
    char *filnam;

    ret = 0;
    filnam = NULL;

    if (mode == 0) {
	if(debflg == 0)
	    filnam = ANL_NUMBERS_LIST;
	else
	    filnam = ANL_NUMS_LST;
    } else if (mode == 1) {
	if(debflg == 0)
	    filnam = ANL_AREAS_LIST;
	else
	    filnam = ANL_AREA_LST;
    } else {
	if(debflg == 0)
	    filnam = ANL_REGIONS_LIST;
	else
	    filnam = ANL_REGS_LST;
    }

    if ((lstfil = fopen(filnam, "r")) == NULL) {
	wrtsysl("cant open phonebook file", filnam, 1);
	return(-1);
    }

    if (fseek(lstfil, (long) offs, SEEK_SET) != 0) {
	wrtsysl("error in phonebook file", filnam, 1);
	ret = -1;
    } else {
	if (fread(name, 1, len + 1, lstfil) != len + 1) {
	    wrtsysl("error in phonebook file", filnam, 1);
	    name[0] = 0;
	    ret = -1;
	} else {
	    name[ANL_NAME_MAX] = 0;
	}
    }

    fclose(lstfil);
    lstfil = NULL;

    return(ret);
}

/*  Lookup name in database  */
int dblookup(char *num, char *name, int mode) 
{
    int entries;
    char *blres, *filnam;
    struct phnum key, *res;
    struct phidx idxkey, *idxres;
    struct stat sb;

    entries = 0;
    memset(&idxkey, 0, sizeof(idxkey));
    memset(&key, 0, sizeof(key));
    memset(&sb, 0, sizeof(sb));
    blres  = NULL;	/*  Blocklist result   */
    idxres = NULL;	/*  Index result  */
    filnam = NULL;
    res    = NULL;	/*  Number result  */

    if (num == NULL) {
	/*  Invalid name blocklist lookup  */
	if(debflg == 0)
	    filnam = ANL_BLOCK_FILE;
	else
	    filnam = ANL_BLCK_FIL;
    } else if (compflg == 0) {
	/*  No compact files  */
	if (mode == 0) {
	    /*  Lookup name  */
	    if(debflg == 0)
		filnam = ANL_NUMBERS_FILE;
	    else
		filnam = ANL_NUMS_FIL;
	} else if (mode == 1) {
	    /*  Lookup area  */
	    if(debflg == 0)
		filnam = ANL_AREAS_FILE;
	    else
		filnam = ANL_AREA_FIL;
	} else {
	    /*  Lookup region  */
	    if(debflg == 0)
		filnam = ANL_REGIONS_FILE;
	    else
		filnam = ANL_REGS_FIL;
 	}
    } else {
	/*  Compact files; Use index to file entry  */
	if (mode == 0) {
	    /*  Lookup name  */
	    if(debflg == 0)
		filnam = ANL_NUMBERS_INDEX;
	    else
		filnam = ANL_NUMS_IDX;
	} else if (mode == 1) {
	    /*  Lookup area  */
	    if(debflg == 0)
		filnam = ANL_AREAS_INDEX;
	    else
		filnam = ANL_AREA_IDX;
	} else {
	    /*  Lookup region  */
	    if(debflg == 0)
		filnam = ANL_REGIONS_INDEX;
	    else
		filnam = ANL_REGS_IDX;
	}
    }

    /*  Open name or numbers file  */
    if ((dbfd = open(filnam, O_RDONLY)) < 0) {
	/*  agi-nameblock.db is optional  */
	if(num != NULL)
	    wrtsysl("cant open phonebook file", filnam, 1);
	dbfd = 0;
	return(-1);
    }

    if (fstat(dbfd, &sb) != 0) {
	wrtsysl("fstat failed", NULL, 1);
	close(dbfd);
	dbfd = 0;
        return(-1);
    }
    dbsiz = (size_t) sb.st_size;
    if (num == NULL)
	entries = (int) (dbsiz / ANL_NAME_SIZ);
    else if (compflg == 0)
	entries = (int) (dbsiz / sizeof(struct phnum));
    else
	 entries = (int) (dbsiz / sizeof(struct phidx));
    /*  Read name or numbers file into memory  */
    dbaddr = mmap(NULL, dbsiz, PROT_READ, MAP_PRIVATE, dbfd, 0);
    if (dbaddr == MAP_FAILED) {
	wrtsysl("mmap failed", NULL, 1);
	dbaddr = NULL;
	close(dbfd);
	dbfd = 0;
	return(-1);
    }

    /* Find name in list  */
    if (debflg != 0) {
	if (num == NULL)
	    fprintf(flog, "Debug: dblookup(): name: %s\n", name);
	else
	    fprintf(flog, "Debug: dblookup(): num: %s\n", num);
	fflush(flog);
    }
    /* void *bsearch(const void *key, const void *base, size_t nmemb, \
      size_t size, int (*compar)(const void *, const void *)); */
    if (num == NULL) {
	/*  Lookup name in blacklist  */
	blres = bsearch(name, dbaddr, entries, ANL_NAME_SIZ, compnam);
	if (blres != NULL) {
	    wrtsysl("Invalid", blres, 0);
	    name[0] = 0;
	}
    } else if (compflg == 0) {
	/*  Lookup number  */
	strcpy(key.num, num);
	res = bsearch(&key, dbaddr, entries, sizeof(struct phnum), compnum);
	if (res != NULL) {
	    strncpy(name, res->name, ANL_NAME_MAX);
	    name[ANL_NAME_MAX] = 0;
	    wrtsysl("dblookup", name, 0);
	}
    } else {
	/*  Lookup number in index  */
	strcpy(idxkey.num, num);
	idxres = bsearch(&idxkey, dbaddr, entries, sizeof(struct phidx), compidx);
	if (idxres != NULL) {
	    if (idxres->namlen < 1 || idxres->namlen > ANL_NAME_MAX) {
		wrtsysl("error in index file", filnam, 1);
	    } else if (idxlookup(idxres->offs, idxres->namlen, name, mode) == 0) {
		wrtsysl("dblookup", name, 0);
	    }
	}
    }

    /*  Done  */
    munmap(dbaddr, dbsiz);
    dbaddr = NULL;
    close(dbfd);
    dbfd = 0;

    return(0);
}

/*  Find name in line  */
int findname(char *line, int len)
{
    int i, j;

    i = 0;
    j = 0;
    while(i < len) {
	if (strncmp(line + i, "  ", 2) == 0) {
	    /*  Wide whitespace  */
	    i += 2;
	    break;
	}
	i++;
    }
    while (i < len) {
	if (line[i] > ' ' || line[i] < 0) {
	    /*  End of whitespace  */
	    j = i;
	    break;
	}
	i++;
    }
    while (i < len) {
	if (strncmp(line + i, "  ", 2) == 0) {
	    /*  Wide whitespace  */
	    line[i] = 0;
	    if (j > 0 && j < i) {
		if (i - j > ANL_NAME_MAX) {
		    /*  Truncate  */
		    line[j + ANL_NAME_MAX] = 0;
		}
		/*  Position of name in line  */
		return(j);
	    }
	    break;
	}
	i++;
    }

    /*  Not found  */
    return(-1);
}

/*  Lookup name on web  */
void weblookup(char *num, char *name, char *buf, char *line)
{
    char comnd[256];
    int inbuf, inp, outp, ret;

    inbuf = 0;
    inp = 0;
    outp = 0;
    ret = 0;
    buf[0] = 0;
    line[0] = 0;
    name[0] = 0;
    memset(comnd, 0, 256);

    /*  Check for starts with '+'  */
    if (num[0] == '+') {
	/*  Escape '+' to '%2B'  */
	snprintf(comnd, 255, "%s \"%s%%2B%s%s\"", \
	  ANL_W3M_COMND, ANL_URL_PREFIX, num + 1, ANL_URL_SUFFIX);
    } else {
	snprintf(comnd, 255, "%s \"%s%s%s\"", \
	  ANL_W3M_COMND, ANL_URL_PREFIX, num, ANL_URL_SUFFIX);
    }

    /*  Open process  */
    pweb = popen(comnd, "r");
    if (pweb == NULL) {
	wrtsysl("cant open w3m", NULL, 1);
	fclose(flog);
	alarm(0);
	signal(SIGALRM, SIG_DFL);
	exit(1);
    }

    if (debflg != 0) {
	/*  Log  */
	fprintf(flog, "Debug: webookup(): num: %s\n", num);
	fflush(flog);
    }

    /*  Read process  */
    outp = 0;
    while ((inbuf = fread(buf, 1, 4095, pweb)) > 0) {
	/*  Find lines of text  */
	inp = 0;
	while (inp < inbuf) {
	    line[outp] = buf[inp];
	    if (line[outp] == 10 || outp > 4092) {
		/*  A line or lots of data  */
		outp++;
		line[outp] = 0;
		if (debflg != 0) {
		    fprintf(flog, "Debug: weblookup(): %s", line);
		    fflush(flog);
		}
		if (name[0] == 0 && line[0] == '+') {
		    /*  Number  */
		    ret = findname(line, outp);
		    if (ret > 0) {
			/*  Found name  */
			strcpy(name, line + ret);
			wrtsysl("weblookup", name, 0);
		    }
		}
		outp = 0;
		line[outp] = 0;
	    } else {
		outp++;
	    }
	    inp++;
	}
    }

    pclose(pweb);
    pweb = NULL;
}

/*
*  Read one or more lines from stdin
*  This may work with fgets().
*  But does it work with fgets() all the time?
*  Better safe then sorry.
*/
int readstr(char *buf, char *line, char *num, char *name)
{
    int cont, inbuf, inp, outp;

    inbuf = 0;		/*  Bytes in buf  */
    inp = 0;		/*  Current position in input buffer  */
    outp = 0;		/*  Current position in output buffer  */
    buf[0] = 0;
    line[0] = 0;

    /*
    *  When both num and name are not NULL;
    *  - Read until empty line.
    *  - Find name and number.
    *  Else;
    *  - Read one line.
    */
    if(num != NULL)
	num[0] = 0;
    if(name != NULL)
	name[0] = 0;

    cont = 1;		/*  Continue while not zero  */

    while (cont != 0) {
	inbuf = read(0, buf, 4095);
	if (inbuf < 0) {
	    if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK) {
		inbuf = 0;
		continue;
	    } else {
		wrtsysl("cant read", NULL, 1);
		break;
	    }
	} else if (inbuf == 0) {
	    break;
	}
	/*  Find lines of text  */
	inp = 0;
	buf[inbuf] = 0;
	while (inp < inbuf) {
	    line[outp] = buf[inp];
	    if (line[outp] == 10) {
		/*  A line  */
		outp++;
		line[outp] = 0;
		if (num == NULL || name == NULL) {
		    /*  Read a single line  */
		    cont = 0;	/*  Stop outer loop  */
		    break;
		}
		if(debflg != 0)
		    fprintf(flog, "%s", line);
		/*  Evaluate contents of line  */
		if (strncasecmp(line, "agi_callerid: ", 14) == 0) {
		    strncpy(num, line + 14, ANL_NUM_MAX);
		    num[ANL_NUM_MAX] = 0;
		} else if (strncasecmp(line, "agi_calleridname: ", 18) == 0) {
		    strncpy(name, line + 18, ANL_NAME_MAX);
		    name[ANL_NAME_MAX] = 0;
		} else if (line[0] == 10 || line[0] == 13) {
		    /*  Empty line: Done  */
		    cont = 0;	/*  Stop outer loop  */
		    break;
		}
		outp = 0;
		line[outp] = 0;
	    } else {
		outp++;
		line[outp] = 0;
	    }
	    if (outp > 4092) {
		wrtsysl("line too long", NULL, 1);
		return(-1);
	    }
	    inp++;
	}
    }

    return(0);
}

/*  Convert name to lower case  */
int namtolow(char *name, char *lnam)
{
    int i;
    size_t wlen;
    wchar_t wnam[ANL_NAME_SIZ];

    i = 0;
    wlen = 0;
    memset(wnam, 0, sizeof(wnam));

    if(debflg != 0)
	fprintf(flog, "Debug: namtolow() name: %s\n", name);

    /*  Convert to wide characters  */
    if ((wlen = mbstowcs(wnam, name, ANL_NAME_MAX)) < 0) {
	wrtsysl("namtolow()", "mbstowcs() failed", 1);
	return(-1);
    }
    /*  Convert to lower case  */
    for(i = 0; i < wlen; i++)
	wnam[i] = towlower(wnam[i]);
    /*  Convert to UTF-8  */
    if (wcstombs(lnam, wnam, ANL_NAME_MAX) < 0) {
	wrtsysl("namtolow()", "mbstowcs() failed", 1);
	return(-1);
    }

    if(debflg != 0)
	fprintf(flog, "Debug: namtolow() converted to: %s\n", lnam);

    return(0);
}

void hlp(void)
{
    fprintf(stderr, "Asterisk name and location lookup AGI\n");
    fprintf(stderr, \
      "Usage: namelookup.agi [-d] [-h] [-l logfile] [-u home] [-v]\n");
    fprintf(stderr, "-d: Debug on.\n");
    fprintf(stderr, "-h: Print this Help and exit.\n");
    fprintf(stderr, "-l: Log file. Default: %s\n", ANL_LOG_FILE);
    fprintf(stderr, "-u: Home directory. Default: %s\n", ANL_W3M_HOME);
    fprintf(stderr, "-v: Print version and exit.\n\n");
}

void prtvers(void)
{
    printf("Version: %s\n", version);
    #ifdef ANL_WITH_WIDE_NAMES
     printf("With name length %d bytes\n", ANL_NAME_MAX);
    #endif
    printf("\n");
}


int main(int argc, char **argv)
{
    char buf[4096], home[256], line[4096], lognam[256], outstr[96];
    char lnam[ANL_NAME_SIZ], name[ANL_NAME_SIZ], num[ANL_NUM_SIZ];
    int disweb, i, len, lktyp, opt;
    FILE *confil;

    char *lookups[4] = {"DbLk", "WbLk", "ArLk", "RgLk"};

    compflg = 0;	/*  Compact files  */
    dbfd    = 0;	/*  Name lookup DataBase FD  */
    dbsiz   = 0;	/*  Size of database  */
    debflg  = 0;	/*  Debug on when not zero  */
    disweb  = 0;	/*  Disable web lookups when not zero  */
    i       = 0;
    len     = 0;
    lktyp   = 0;	/*  Lookup type  */
    opt     = 0;
    dbaddr = NULL;	/*  Location of database in memory  */
    flog   = NULL;	/*  Log file  */
    lstfil = NULL;	/*  List file  */
    pweb   = NULL;	/*  Browser process  */
    memset(buf,  0, 4096);
    memset(home, 0, 256);		/*  Alt home directory  */
    memset(line, 0, 4096);
    memset(name, 0, ANL_NAME_SIZ);
    memset(lnam, 0, ANL_NAME_SIZ);	/*  Lower case version of name  */
    memset(lognam, 0, 256);		/*  Alt log file name  */
    memset(num,  0, ANL_NUM_SIZ);	/*  Phone number  */
    memset(outstr, 0, 96);		/*  String send to Asterisk  */
    confil = NULL;

    /*  mbstowcs() and wcstombs() need this  */
    setlocale(LC_ALL, "C.UTF-8");

    wrtsysl("Version", version, 0);
    #ifdef ANL_WITH_WIDE_NAMES
     wrtsysl("With long names", NULL, 0);
    #endif

    while ((opt = getopt(argc, argv, "dhl:u:v")) != -1) {
	switch (opt) {
	    case 'd':
		debflg = 1;
		wrtsysl("Debug on", NULL, 0);
		break;
	    case 'h':
		hlp();
		exit(0);
	    case 'l':
		strncpy(lognam, optarg, 255);
		break;
	    case 'u':
		strncpy(home, optarg, 255);
		break;
	    case 'v':
		prtvers();
		exit(0);
	    default:
		hlp();
		exit(1);
	}
    }

    if (debflg == 0)
	confil = fopen(ANL_CONFIG_FILE, "r");
    else
	confil = fopen(ANL_CONF_FIL, "r");
    if (confil != NULL) {
	while (fgets(line, 4096, confil)) {
	    if (strncasecmp(line, "weblookup=", 10) == 0) {
		if (line[10] == '0' || \
		  strncasecmp(line + 10, "no", 2) == 0 || \
		  strncasecmp(line + 10, "off", 3) == 0 || \
		  strncasecmp(line + 10, "false", 5) == 0) {
		    /*  Disable web lookups  */
		    disweb = 1;
		} else if (line[10] == '1' || \
		  strncasecmp(line + 10, "yes", 3) == 0 || \
		  strncasecmp(line + 10, "on", 2) == 0 || \
		  strncasecmp(line + 10, "true", 4) == 0) {
		    /*  Enable web lookups (default)  */
		    disweb = 0;
		}
	    } else if (strncasecmp(line, "compact=", 8) == 0) {
		if (line[8] == '0' || \
		  strncasecmp(line + 8, "no", 2) == 0 || \
		  strncasecmp(line + 8, "off", 3) == 0 || \
		  strncasecmp(line + 8, "false", 5) == 0) {
		    /*  Disable compact files  */
		    compflg = 0;
		} else if (line[8] == '1' || \
		  strncasecmp(line + 8, "yes", 3) == 0 || \
		  strncasecmp(line + 8, "on", 2) == 0 || \
		  strncasecmp(line + 8, "true", 4) == 0) {
		    /*  Enable compact files  */
		    compflg = 1;
		}
	    }
	}
	if (debflg != 0) {
	    if(disweb != 0)
		wrtsysl("Web lookups disabled", NULL, 0);
	    if(compflg != 0)
		wrtsysl("Compact files enabled", NULL, 0);
	}

	fclose(confil);
	line[0] = 0;
    }

    if (debflg == 0 && disweb == 0) {
	/*
	*  Set w3m homedir
	*  Without this w3m wants to write in /.w3m/
	*/
	if (home[0] == 0) {
	    /*  Default home dir  */
	    setenv("HOME", ANL_W3M_HOME, 1);
	    chdir(ANL_W3M_HOME);
	} else {
	    /*  Home dir set with -u  */
	    setenv("HOME", home, 1);
	    chdir(home);
	}
    }

    signal(SIGALRM, timeout);
    alarm(ANL_MAX_TIM);

    /*  Open log  */
    if (debflg == 0) {
	if (lognam[0] == 0) {
	    /*  Default log file  */
	    flog = fopen(ANL_LOG_FILE, "a");
	} else {
	    /*  Log file set with -l  */
	    flog = fopen(lognam, "a");
	}
    } else {
	/*  Log file in default dir  */
	flog = fopen(ANL_LOG_FIL, "a");
    }
    if (flog == NULL) {
	if (debflg == 0) {
	    if(lognam[0] == 0)
		wrtsysl("Can't open", ANL_LOG_FILE, 1);
	    else
		wrtsysl("Can't open", lognam, 1);
	} else {
	    wrtsysl("Can't open", ANL_LOG_FIL, 1);
	}
	alarm(0);
	signal(SIGALRM, SIG_DFL);
	exit(1);
    }

    /*
    *  Read from stdin / inetd
    *  Find name and number
    */
    if (readstr(buf, line, num, name) != 0) {
	/*  Error  */
	fclose(flog);
	alarm(0);
	signal(SIGALRM, SIG_DFL);
	exit(0);
    }

    /*  Cleanup number  */
    i = 0;
    while (i < ANL_NUM_SIZ) {
	if (num[i] < ' ' && num[i] > -1) {
	    num[i] = 0;
            break;
	}
	i++;
    }
    len = i;	/*  Length of number  */
    i = 0;
    /*  Cleanup name  */
    while (i < ANL_NAME_SIZ) {
	if (name[i] < ' ' && name[i] > -1) {
	    name[i] = 0;
	    break;
	}
	i++;
    }

    if (debflg != 0) {
	if(num[0] != 0)
	    fprintf(flog, "Debug: Found num: %s\n", num);
	if(name[0] != 0)
	    fprintf(flog, "Debug: Found name: %s\n", name);
	fflush(flog);
    }

    if (name[0] != 0) {
	/*  Convert name to lower case and lookup  */
	if (namtolow(name, lnam) == 0 && dblookup(NULL, lnam, 0) == 0) {
	    /*  namtolow() OK and dblookup() OK  */
	    if (lnam[0] == 0) {
		/*  lnam found and thus cleared  */
		name[0] = 0;
	    }
	} else {
	    /*  namtolow() failed or no name bloklist file  */
	    if (strcasecmp(name, "unknown") == 0 || \
	      strcasecmp(name, "Privacy Manager") == 0 || \
	      strcasecmp(name, "Anonymous") == 0) {
		/*  Clear  */
		wrtsysl("Invalid", name, 0);
		name[0] = 0;
	    }
	}
    }

    if(name[0] != 0)
	wrtsysl("Name from VP", name, 0);

    /*  Check for number and no name  */
    if (num[0] == 0 || name[0] != 0) {
	fclose(flog);
	alarm(0);
	signal(SIGALRM, SIG_DFL);
	exit(0);
    }

    /*  Check for number; +0123456789  */
    i = 0;
    while (i < len) {
	if ((num[i] < '0' || num[i] > '9') && num[i] != '+') {
	    fclose(flog);
	    alarm(0);
	    signal(SIGALRM, SIG_DFL);
	    exit(0);
	}
	i++;
    }

    /*  Write number to syslog  */
    wrtsysl("Lookup nr", num, 0);

    /*  Lookup name in database  */
    lktyp = 0;
    dblookup(num, name, 0);
    if (name[0] == 0 && disweb == 0) {
	/*  Not found; Do a web lookup  */
	lktyp = 1;
	weblookup(num, name, buf, line);
    }
    if (name[0] == 0) {
	/*  Not found; Do a area code lookup  */
	lktyp = 2;
	dblookup(num, name, 1);
    }
    if (name[0] == 0) {
	/*  Not found; Do a region code lookup  */
	lktyp = 3;
	dblookup(num, name, 2);
    }

    if (name[0] != 0) {
	/*  Note: This really needs a linefeed!  */
	if (lktyp > 1) {
	    /*  Geographic  */
	    snprintf(outstr, 95, "SET VARIABLE NAME \"Lc %s\"\n", name);
	} else {
	    /*  Name  */
	    snprintf(outstr, 95, "SET VARIABLE NAME \"%s\"\n", name);
	}
	wrtstr(outstr);
	/*  Write to log  */
	wrtlogtim(flog);
	fprintf(flog, "%s\t%s\t%s\t", lookups[lktyp], num, name);
	if (debflg == 0) {
	    readstr(buf, line, NULL, NULL);
	    fprintf(flog, "%s", line);
	} else {
	    fprintf(flog, "Debug\n");
	}
	fflush(flog);
    }

    if(pweb != NULL)
	pclose(pweb);
    fclose(flog);

    alarm(0);
    signal(SIGALRM, SIG_DFL);

    return(0);
}

